# Process-Based Simulation with Stackless Coroutines
## Overview
This repository contains the source code for the simulator core discussed in our SAM 2020 paper.
In order to run the examples, you should install the [Rust-compiler] first.

The `src/` directory contains the source code for our simulator core *simcore-rs* as well as the decompressor-tokenizer-
and the barbershop-example from the paper. The folder `benches/` contains the code used for benchmarking the barbershop-
example. In `slx/` one can find the SLX-code for the barbershop-example, including benchmarking provisions.
The `results/` directory contains our benchmarking results for the barbershop-example in Rust and SLX.

Unfortunately, we are unable to provide a link to the SLX-compiler at this time. Please contact us if you wish to
execute the barbershop-example in SLX as well.

## Executing the Sample Code
You may execute the decompressor-tokenizer example using `cargo run --bin coroutines` and the barbershop example
using `cargo run --bin barbershop`. For the benchmarks of the simulator core in case of the barbershop example you may
enter `cargo bench` into your terminal. Cargo will download and compile all the dependencies for you.

Should you wish to execute the SLX code as well, you will need to obtain a version of SLX first. The free student
version is sufficient to run the benchmarks with a sample size of 1 for a quick comparison between SLX and Rust, but in
order to get statistically relevant results, you need to own a license for the full version of SLX.

## Benchmarks
We executed the benchmarks under Windows 10 on an Intel Core i7-10750H processor with 6 cores and 32 GB DDR4 RAM.
You can find the results under `results/barbershop.lis` for SLX and `results/barbershop/report/index.html` for Rust.
In a nutshell: we see a speed difference between the SLX- and the Rust-version of the barbershop example of about 3 in
favor of SLX. We have summarized the results in the following table for your convenience:

| Model Time | SLX Mean |  SLX SD  | Rust Mean |  Rust SD |
|:----------:|---------:|---------:|----------:|---------:|
| 100000     | 0.514 ms | 0.052 ms |  1.446 ms | 0.039 ms |
| 200000     | 1.034 ms | 0.069 ms |  2.882 ms | 0.046 ms |
| 300000     | 1.552 ms | 0.073 ms |  4.389 ms | 0.064 ms |
| 400000     | 2.069 ms | 0.089 ms |  5.754 ms | 0.051 ms |
| 500000     | 2.592 ms | 0.116 ms |  7.188 ms | 0.054 ms |
| 600000     | 3.106 ms | 0.121 ms |  8.904 ms | 0.117 ms |
| 700000     | 3.618 ms | 0.127 ms | 10.232 ms | 0.078 ms |
| 800000     | 4.136 ms | 0.150 ms | 11.587 ms | 0.230 ms |

[Rust-compiler]: https://rust.sh
